// Trainer.cpp
//

#include "Trainer.h"
#include "Athlete.h"
#include "DataVisitor.h"
#include "assert.h"

Trainer::Trainer (std::string name): Boxer(name){
	pose.leftGuard_=false;
	pose.rightGuard_=true;
	pose.hit_=false;
	pose.leftPunching_=false;
	pose.rightPunching_=false;
	hitTime = (double)clock();
	pose.setChanged();
}

void Trainer::acceptExporter(const DataVisitor &visitor){

}

const BoxerPose* Trainer::getPose() const{
	return &pose;
}
const std::string& Trainer::getFeedback () const{
	return feedback; 
}
void Trainer::clearFeedback (){
	feedback.clear();
}

void Trainer::update (Observable * o, Argument * arg){
	assert(o);
	const Athlete *athlete = dynamic_cast<Athlete*>(o);
	assert(athlete);

	PlayerData data(athlete->leftGuard(), 
		athlete->rightGuard(), 
		athlete->eyesOnOpponent);

	playerDataSet.push_back(data);

	if(pose.hit() && (double)clock() > hitTime + (1 * CLOCKS_PER_SEC) ){
		if(pose.leftGuard()){
			pose.leftGuard_=false;
			pose.rightGuard_=true;
		}else{
			pose.leftGuard_=true;
			pose.rightGuard_=false;
		}
		pose.hit_ = false;
		pose.setChanged();
	}

	if((pose.rightPunching() || pose.leftPunching()) && (double)clock() > punchTime + (1 * CLOCKS_PER_SEC) ){
		pose.leftPunching_=false;
		pose.rightPunching_=false;

		pose.hit_ = false;
		pose.setChanged();
	}
	if(athlete->leftPunching()&& !pose.rightGuard()||
					athlete->rightPunching() && !pose.leftGuard() ){
		pose.hit_ = true;
		hitTime = (double)clock();
		pose.setChanged();
		if(feedback.find("Nice shot!") == string::npos){
			feedback.append("Nice shot!\n");
		}
	}

	int leftGuargCount=0;
	int rightGuargCount=0;
	int eyesDownCount=0;

	std::list<PlayerData>::reverse_iterator it = playerDataSet.rbegin();
	for(int i=0; i<12; i++){
		if(!it->leftGuard){
			leftGuargCount++;
		}
		if(!it->rightGuard ){
			rightGuargCount++;
		}
		if(!it->eyesOnOpponent){
			eyesDownCount++;
		}

		it++;
		if(it == playerDataSet.rend()){
			break;
		}
	}

	if(leftGuargCount>5 && feedback.find("Keep your left") == string::npos){
		feedback.append("Keep your left\n guard up!\n");
	}else if(leftGuargCount>10){
		punchTime = (double)clock();
		pose.leftGuard_=true;
		pose.rightGuard_=false;
		pose.hit_=false;
		pose.leftPunching_=false;
		pose.rightPunching_=true;
		pose.setChanged();
	}

	if(rightGuargCount>5 && feedback.find("Keep your right") == string::npos){
		feedback.append("Keep your right\n guard up!\n");
	}else if(rightGuargCount>10){
		punchTime = (double)clock();
		pose.leftGuard_=false;
		pose.rightGuard_=true;
		pose.hit_=false;
		pose.leftPunching_=true;
		pose.rightPunching_=false;
		pose.setChanged();

	}

	if(!athlete->eyesOnOpponent && feedback.find("Keep your eyes") == string::npos){
		feedback.append("Keep your eyes\n on your opponent!\n");
	}

	pose.notifyObservers(NULL);
}
Trainer::~ Trainer (){}
