#include "TrainerAnimation.h"
#include <osgDB/ReadFile>
#include <osgDB/Registry>
#include <osg/Texture2D>
#include "Trainer/TrainerPose.h"

#include <assert.h>
void TrainerAnimation::init(){
	billboard = new osg::Billboard();
	billboard->setMode(osg::Billboard::AXIAL_ROT);
    billboard->setAxis(osg::Vec3(0.0f,1.0f,0.0f));
    billboard->setNormal(osg::Vec3(1.0f,0.0f,0.0f));
	billboard->setDataVariance(osg::Object::DYNAMIC);

	leftGuardDown =osgDB::readImageFile("JohnLeftDown.tga");
	rightGuardDown =osgDB::readImageFile("JohnRightDown.tga");
	leftPunching =osgDB::readImageFile("JohnLeftPunching.tga");
	rightPunching =osgDB::readImageFile("JohnRightPunching.tga");
	leftGuardDownAndHit =osgDB::readImageFile("JohnLeftDownHit.tga");
	rightGuardDownAndHit =osgDB::readImageFile("JohnRightDownHit.tga");
	reset =osgDB::readImageFile("John_neutral.tga");
	billboardImage=new osg::Image();
	billboardImage->allocateImage(reset->s(),
		reset->t(), reset->r(), reset->getPixelFormat(),reset->getDataType());
	billboardImage->setImage(reset->s(), reset->t(), reset->r(),reset->getInternalTextureFormat(),
		reset->getPixelFormat(),reset->getDataType(),reset->data(),osg::Image::NO_DELETE, 1);

	quadImage = createSquare(osg::Vec3(0.0f, -240.0f,-108.0f)
		,osg::Vec3(0.0f,288.0f,0.0f),osg::Vec3(0.0f,0.0f,216.0f),billboardImage);
    billboard->addDrawable(quadImage, osg::Vec3(0.0f,0.0f,0.0f));
	
	assert(billboardImage);
	assert(leftGuardDown);
	assert(rightGuardDown);
	assert(leftPunching);
	assert(rightPunching);
	assert(leftGuardDownAndHit);
	assert(rightGuardDownAndHit);
	assert(reset);
	assert(quadImage);
	assert(stateset);
}

void TrainerAnimation::update (Observable * o, Argument * arg){
	assert(o);
	const TrainerPose *trainerPose = dynamic_cast<TrainerPose*>(o);
	assert(trainerPose);	
    double setImageTime = (double)clock();
	if(trainerPose->leftPunching()){
		billboardImage->setImage(leftPunching->s(), leftPunching->t(), leftPunching->r(),leftPunching->getInternalTextureFormat(),
			leftPunching->getPixelFormat(),leftPunching->getDataType(),leftPunching->data(),osg::Image::NO_DELETE, 1);
	}else if(trainerPose->rightPunching()){
		billboardImage->setImage(rightPunching->s(), rightPunching->t(), rightPunching->r(),rightPunching->getInternalTextureFormat(),
			rightPunching->getPixelFormat(),rightPunching->getDataType(),rightPunching->data(),osg::Image::NO_DELETE, 1);
	}else if(!trainerPose->leftGuard()){
		if(trainerPose->hit()){
			billboardImage->setImage(leftGuardDownAndHit->s(), leftGuardDownAndHit->t(), leftGuardDownAndHit->r(),leftGuardDownAndHit->getInternalTextureFormat(),
				leftGuardDownAndHit->getPixelFormat(),leftGuardDownAndHit->getDataType(),leftGuardDownAndHit->data(),osg::Image::NO_DELETE, 1);
		}else{
			billboardImage->setImage(leftGuardDown->s(), leftGuardDown->t(), leftGuardDown->r(),leftGuardDown->getInternalTextureFormat(),
				leftGuardDown->getPixelFormat(),leftGuardDown->getDataType(),leftGuardDown->data(),osg::Image::NO_DELETE, 1);
		} 
	}else if(!trainerPose->rightGuard()){
		if(trainerPose->hit()){
			billboardImage->setImage(rightGuardDownAndHit->s(), rightGuardDownAndHit->t(), rightGuardDownAndHit->r(),rightGuardDownAndHit->getInternalTextureFormat(),
				rightGuardDownAndHit->getPixelFormat(),rightGuardDownAndHit->getDataType(),rightGuardDownAndHit->data(),osg::Image::NO_DELETE, 1);
		}else{
			billboardImage->setImage(rightGuardDown->s(), rightGuardDown->t(), rightGuardDown->r(),rightGuardDown->getInternalTextureFormat(),
				rightGuardDown->getPixelFormat(),rightGuardDown->getDataType(),rightGuardDown->data(),osg::Image::NO_DELETE, 1);
		}
	}else{
		billboardImage->setImage(reset->s(), reset->t(), reset->r(),reset->getInternalTextureFormat(),
			reset->getPixelFormat(),reset->getDataType(),reset->data(),osg::Image::NO_DELETE, 1);
	}
}

TrainerAnimation::TrainerAnimation(void){
	billboard=NULL;
	billboardImage=NULL;
	reset=NULL;
	leftPunching=NULL;
	rightPunching=NULL;
	leftGuardDown=NULL;
	rightGuardDown=NULL;
	leftGuardDownAndHit=NULL;
	rightGuardDownAndHit=NULL;
	quadImage=NULL;
	stateset=NULL;
}

osg::Drawable * TrainerAnimation::createSquare (osg::Vec3 const & corner, osg::Vec3 const & width, osg::Vec3 const & height, osg::Image * image)
{
    // set up the Geometry.
    osg::Geometry* geom = new osg::Geometry;

    osg::Vec3Array* coords = new osg::Vec3Array(4);
    (*coords)[0] = corner;
    (*coords)[1] = corner+width;
    (*coords)[2] = corner+width+height;
    (*coords)[3] = corner+height;


    geom->setVertexArray(coords);

    osg::Vec3Array* norms = new osg::Vec3Array(1);
    (*norms)[0] = width^height;
    (*norms)[0].normalize();
    
    geom->setNormalArray(norms);
    geom->setNormalBinding(osg::Geometry::BIND_OVERALL);

    osg::Vec2Array* tcoords = new osg::Vec2Array(4);
    (*tcoords)[0].set(0.0f,1.0f);
    (*tcoords)[1].set(0.0f,0.0f);
    (*tcoords)[2].set(1.0f,0.0f);
    (*tcoords)[3].set(1.0f,1.0f);

    geom->setTexCoordArray(0,tcoords);
    
    geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::QUADS,0,4));
    
    if (image){
		stateset = new osg::StateSet;
        osg::Texture2D* texture = new osg::Texture2D(billboardImage);
		texture->setDataVariance(osg::Object::DYNAMIC);
		stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
		stateset->setMode(GL_BLEND,osg::StateAttribute::ON);
		stateset->setDataVariance(osg::Object::DYNAMIC);
        geom->setStateSet(stateset);
    }
	geom->setDataVariance(osg::Object::DYNAMIC);

    assert(geom);
    return geom;
}

osg::Geode *TrainerAnimation::getGeode(){
	return billboard;
}

TrainerAnimation::~TrainerAnimation (){
}